------------------------------
--      Are you local?      --
------------------------------

local L = AceLibrary("AceLocale-2.2"):new("ImprovedRWF")
local PLAYER = UnitName("player")


local function __DoNothing() end
local _AddMessage, _IsSpam, _PlaySound


local backdrops = {
	["chatframe"] = {
		bgFile = "Interface/ChatFrame/ChatFrameBackground",
		edgeFile = "Interface/ChatFrame/ChatFrameBackground",
		edgeSize = 2,
		tile = false,
		insets = { left = 0, right = 0, top = 0, bottom = 0 },
	},
	["tooltip"] = {
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		edgeSize = 16,
		tile = true,
		tileSize = 32,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	},
}

local fonts = {}
local fontsizes = {}
fonts["default"], fontsizes["huge"], _ = GameFontNormalHuge:GetFont()
_, fontsizes["large"], _ = GameFontNormalLarge:GetFont()
_, fontsizes["normal"], _ = GameFontNormal:GetFont()
_, fontsizes["small"], _ = GameFontNormalSmall:GetFont()

local fontoutlines = {
	["none"] = "",
	["normal"] = "OUTLINE",
	["thick"] = "THICKOUTLINE",
}


---------------------------------
--      Addon Declaration      --
---------------------------------

ImprovedRWF = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "AceHook-2.1", "FuBarPlugin-2.0")

ImprovedRWF:RegisterDB("ImprovedRWFDB", "ImprovedRWFDBPerChar")
ImprovedRWF.defaults = {
	addname = false,
	colourname = true,
	sound = true,
	supressown = false,
	supressall = false,

	backdrop = false,
	backdropcolours = {
		["chatframe"] = {r = 0, g = 0, b = 0, a = 0.2, br = 0, bg = 0, bb = 0, ba = 0.4},
		["tooltip"] = {	r = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, g = TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, b = TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, a = 1,
						br = TOOLTIP_DEFAULT_COLOR.r, bg = TOOLTIP_DEFAULT_COLOR.g, bb = TOOLTIP_DEFAULT_COLOR.b, ba = 1								},
	},
	backdropstyle = "chatframe",
	font = "default",
	fontsize = "huge",
	fontoutline = "none",
	usecustomcolour = false,
	customcolour = {r = ChatTypeInfo["RAID_WARNING"].r, g = ChatTypeInfo["RAID_WARNING"].g, b = ChatTypeInfo["RAID_WARNING"].b},
	height = 40,
	width = 512,
	move = false,
	timevisible = 10,
}
ImprovedRWF:RegisterDefaults("profile", ImprovedRWF.defaults)

ImprovedRWF.OnMenuRequest = {
	type = "group",
	args = {
		header1 = {
			type = "header",
			name = L["Frame Options"],
			order = 101,
		},
		[L["addname"]] = {
			type = "toggle",
			name = L["Add Name"],
			desc = L["Add player name to raid warning messages"],
			get = function() return ImprovedRWF.db.profile.addname end,
			set = function()
				local v = not ImprovedRWF.db.profile.addname
				ImprovedRWF.db.profile.addname = v
				_AddMessage = ImprovedRWF:GenerateAddMessage()
			end,
			order = 102,
		},
		[L["colourname"]] = {
			type = "toggle",
			name = L["Colour Name"],
			desc = L["Colour player name according to the class"],
			get = function() return ImprovedRWF.db.profile.colourname end,
			set = function()
				local v = not ImprovedRWF.db.profile.colourname
				ImprovedRWF.db.profile.colourname = v
				_AddMessage = ImprovedRWF:GenerateAddMessage()
			end,
			disabled = function() return not ImprovedRWF.db.profile.addname end,
			hidden = function() return not Teknicolor end,
			order = 103,
		},
		[L["sound"]] = {
			type = "toggle",
			name = L["Play Sound"],
			desc = L["Play a sound on incoming raid warning messages"],
			get = function() return ImprovedRWF.db.profile.sound end,
			set = function()
				local v = not ImprovedRWF.db.profile.sound
				ImprovedRWF.db.profile.sound = v
				_PlaySound = ImprovedRWF:GeneratePlaySound()
			end,
			order = 104,
		},
		[L["supressall"]] = {
			type = "toggle",
			name = L["Supress All Messages"],
			desc = L["Stop all messages sent by players from appearing in the raid warning frame"],
			get = function() return ImprovedRWF.db.profile.supressall end,
			set = function(v)
				ImprovedRWF.db.profile.supressall = v
				ImprovedRWF.RaidWarningFrame_OnEvent = v and ImprovedRWF._RaidWarningFrame_OnEvent_SupressAll or ImprovedRWF._RaidWarningFrame_OnEvent
			end,
			order = 105,
		},
		[L["supressown"]] = {
			type = "toggle",
			name = L["Supress Own Messages"],
			desc = L["Stop messages sent by yourself from appearing in the raid warning frame"],
			get = function() return ImprovedRWF.db.profile.supressall or ImprovedRWF.db.profile.supressown end,
			set = function()
				local v = not ImprovedRWF.db.profile.supressown
				ImprovedRWF.db.profile.supressown = v
				_IsSpam = ImprovedRWF:GenerateIsSpam()
			end,
			disabled = function() return ImprovedRWF.db.profile.supressall end,
			order = 106,
		},

		blank2 = {
			type = "header",
			order = 201,
		},
		header2 = {
			type = "header",
			name = L["Frame Appearance"],
			order = 202,
		},
		[L["backdrop"]] = {
			type = "group",
			name = L["Backdrop"],
			desc = L["Customise the raid warning frame's backdrop style"],
			args = {
				[L["enable"]] = {
					type = "toggle",
					name = L["Enable"],
					desc = L["Enable backdrop"],
					get = function() return ImprovedRWF.db.profile.backdrop end,
					set = function()
						local v = not ImprovedRWF.db.profile.backdrop
						ImprovedRWF.db.profile.backdrop = v
						ImprovedRWF:SetBackdrop()
					end,
					order = 101,
				},
				[L["style"]] = {
					type = "text",
					name = L["Style"],
					desc = L["Select the raid warning frame's backdrop style"],
					get = function() return L[ImprovedRWF.db.profile.backdropstyle] end,
					set = function(v)
						-- currently AceLocale-2.1 doesn't have reverse translations for the base locale
						if L:HasReverseTranslation(v) then v = L:GetReverseTranslation(v) end
						ImprovedRWF.db.profile.backdropstyle = v
						ImprovedRWF:SetBackdrop()
					end,
					validate = {L["chatframe"], L["tooltip"]},
					disabled = function() return not ImprovedRWF.db.profile.backdrop end,
					order = 102,
				},
				[L["backcolour"]] = {
					type = "color",
					hasAlpha = true,
					name = L["Pick Backdrop Colour"],
					desc = L["Pick a backdrop colour"],
					get = function()
						local colour = ImprovedRWF.db.profile.backdropcolours[ImprovedRWF.db.profile.backdropstyle]
						return colour.r, colour.g, colour.b
					end,
					set = function(r, g, b)
						local colour = ImprovedRWF.db.profile.backdropcolours[ImprovedRWF.db.profile.backdropstyle]
						colour.r, colour.g, colour.b = r, g, b
						RaidWarningFrame:SetBackdropColor(r, g, b, a)
					end,
					disabled = function() return not ImprovedRWF.db.profile.backdrop end,
					order = 103,
				},
				[L["bordercolour"]] = {
					type = "color",
					hasAlpha = true,
					name = L["Pick Border Colour"],
					desc = L["Pick a border colour"],
					get = function()
						local colour = ImprovedRWF.db.profile.backdropcolours[ImprovedRWF.db.profile.backdropstyle]
						return colour.br, colour.bg, colour.bb, colour.ba
					end,
					set = function(r, g, b, a)
						local colour = ImprovedRWF.db.profile.backdropcolours[ImprovedRWF.db.profile.backdropstyle]
						colour.br, colour.bg, colour.bb, colour.ba = r, g, b, a
						RaidWarningFrame:SetBackdropBorderColor(r, g, b, a)
					end,
					disabled = function() return not ImprovedRWF.db.profile.backdrop end,
					order = 104,
				},
			},
			order = 203,
		},
		[L["font"]] = {
			type = "group",
			name = L["Font"],
			desc = L["Customise the raid warning font"],
			args = {
				[L["outline"]] = {
					type = "text",
					name = L["Outline"],
					desc = L["Set the font outline (none, normal or thick)"],
					get = function() return ImprovedRWF.db.profile.fontoutline end,
					set = function(v)
						-- currently AceLocale-2.1 doesn't have reverse translations for the base locale
						if L:HasReverseTranslation(v) then v = L:GetReverseTranslation(v) end
						ImprovedRWF.db.profile.fontoutline = v
						ImprovedRWF:SetFont()
					end,
					validate = {L["none"], L["normal"], L["thick"]},
					disabled = function() return oRA_RaidSay and not ImprovedRWF.db.profile.orsfontsize end,
					order = 101,
				},
				[L["size"]] = {
					type = "text",
					name = L["Size"],
					desc = L["Set the font size (small, normal, large or huge)"],
					get = function() return L[ImprovedRWF.db.profile.fontsize] end,
					set = function(v)
						-- currently AceLocale-2.1 doesn't have reverse translations for the base locale
						if L:HasReverseTranslation(v) then v = L:GetReverseTranslation(v) end
						ImprovedRWF.db.profile.fontsize = v
						ImprovedRWF:SetFont()
					end,
					validate = {L["small"], L["normal"], L["large"], L["huge"]},
					order = 102,
				},
				[L["usecolour"]] = {
					type = "toggle",
					name = L["Use Custom Colour"],
					desc = L["Use a custom font colour"],
					get = function() return ImprovedRWF.db.profile.usecustomcolour end,
					set = function()
						local v = not ImprovedRWF.db.profile.usecustomcolour
						ImprovedRWF.db.profile.usecustomcolour = v
						_AddMessage = ImprovedRWF:GenerateAddMessage()
					end,
					order = 103,
				},
				[L["colour"]] = {
					type = "color",
					name = L["Pick Colour"],
					desc = L["Pick a font colour"],
					get = function()
						local colour = ImprovedRWF.db.profile.customcolour
						return colour.r, colour.g, colour.b
					end,
					set = function(r, g, b)
						local colour = ImprovedRWF.db.profile.customcolour
						colour.r, colour.g, colour.b = r, g, b
					end,
					disabled = function() return not ImprovedRWF.db.profile.usecustomcolour end,
					order = 104,
				},
			},
			order = 204,
		},
		[L["height"]] = {
			type = "range",
			name = L["Frame Height"],
			desc = L["Set the frame height (40 - 800)"],
			get = function() return ImprovedRWF.db.profile.height end,
			set = function(v)
				ImprovedRWF.db.profile.height = v
				RaidWarningFrame:SetHeight(v)
			end,
			min = 40,
			max = 800,
			step = 1,
			isPercent = false,
			order = 205,
		},
		[L["width"]] = {
			type = "range",
			name = L["Frame Width"],
			desc = L["Set the frame width (512 - 2048)"],
			get = function() return ImprovedRWF.db.profile.width end,
			set = function(v)
				ImprovedRWF.db.profile.width = v
				RaidWarningFrame:SetWidth(v)
			end,
			min = 512,
			max = 2048,
			step = 1,
			isPercent = false,
			order = 206,
		},
		[L["move"]] = {
			type = "toggle",
			name = L["Move Frame"],
			desc = L["Allow the raid warning frame to be moved"],
			get = function() return ImprovedRWF.db.profile.move end,
			set = function()
				local v = not ImprovedRWF.db.profile.move
				ImprovedRWF.db.profile.move = v
				ImprovedRWF:AttachToAnchorFrame(v)
				RaidWarningAnchorFrame:Hide()
			end,
			order = 207,
		},
		[L["anchor"]] = {
			type = "toggle",
			name = L["Show Anchor"],
			desc = L["Show the anchor for moving the raid warning frame"],
			get = function() return RaidWarningAnchorFrame:IsShown() end,
			set = function()
				if RaidWarningAnchorFrame:IsShown() then
					RaidWarningAnchorFrame:Hide()
				else
					RaidWarningAnchorFrame:Show()
				end
			end,
			disabled = function() return not ImprovedRWF.db.profile.move end,
			order = 208,
		},
		[L["timevisible"]] = {
			type = "range",
			name = L["Time Visible"],
			desc = L["Set the time a message will be visible (3 - 30)"],
			get = function() return ImprovedRWF.db.profile.timevisible end,
			set = function(v)
				ImprovedRWF.db.profile.timevisible = v
				RaidWarningFrame:SetTimeVisible(v)
			end,
			min = 1,
			max = 30,
			step = 1,
			isPercent = false,
			order = 209,
		},

		blank3 = {
			type = "header",
			order = 301,
		},
		header3 = {
			type = "header",
			name = L["Testing"],
			order = 302,
		},
		[L["test"]] = {
			type = "text",
			name = L["Test Message"],
			desc = L["Display a test raid warning message"],
			get = false,
			set = "Test",
			usage = L["<test message>"],
			order = 303,
		},
	},
}
ImprovedRWF:RegisterChatCommand({ "/ImprovedRWF", "/irwf" }, ImprovedRWF.OnMenuRequest)

ImprovedRWF.hideWithoutStandby = true
ImprovedRWF.hasIcon = true
ImprovedRWF.cannotDetachTooltip = true


------------------------------
--      Initialization      --
------------------------------

function ImprovedRWF:OnInitialize()
	RaidWarningAnchorFrame:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b);
	RaidWarningAnchorFrame:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b);
end

function ImprovedRWF:OnEnable()
	self.RaidWarningFrame_OnEvent = self.db.profile.supressall and self._RaidWarningFrame_OnEvent_SupressAll or self._RaidWarningFrame_OnEvent
	self:Hook("RaidWarningFrame_OnEvent", true)

	_AddMessage = ImprovedRWF:GenerateAddMessage()
	_IsSpam = ImprovedRWF:GenerateIsSpam()
	_PlaySound = ImprovedRWF:GeneratePlaySound()

	RaidWarningFrame:SetBackdrop(backdrops[self.db.profile.backdrop])
	RaidWarningFrame:SetHeight(self.db.profile.height)
	RaidWarningFrame:SetWidth(self.db.profile.width)
	RaidWarningFrame:SetTimeVisible(self.db.profile.timevisible)

	self:AttachToAnchorFrame(self.db.profile.move)
	self:SetBackdrop()
	self:SetFont()
end

function ImprovedRWF:OnDisable()
	RaidWarningFrame:SetBackdrop(nil)
	RaidWarningFrame:SetFontObject(fonts[self.defaults.fontsize] or GameFontNormalHuge)
	RaidWarningFrame:SetHeight(self.defaults.height)
	RaidWarningFrame:SetWidth(self.defaults.width)
	RaidWarningFrame:SetTimeVisible(self.defaults.timevisible)

	self:AttachToAnchorFrame(self.defaults.move)
end


-----------------------
--      Methods      --
-----------------------

function ImprovedRWF:AttachToAnchorFrame(attached)
	if attached then
		RaidWarningFrame:SetPoint("TOP", RaidWarningAnchorFrame, "BOTTOM", 0, 0)
	else
		RaidWarningFrame:SetPoint("TOP", UIErrorsFrame, "BOTTOM", 0, 0)
	end
end

function ImprovedRWF:SetBackdrop()
	if self.db.profile.backdrop then
		local colours = self.db.profile.backdropcolours[self.db.profile.backdropstyle]

		RaidWarningFrame:SetBackdrop(backdrops[self.db.profile.backdropstyle])
		RaidWarningFrame:SetBackdropColor(colours.r, colours.g, colours.b, colours.a)
		RaidWarningFrame:SetBackdropBorderColor(colours.br, colours.bg, colours.bb, colours.ba)
	else
		RaidWarningFrame:SetBackdrop(nil)
	end
end

function ImprovedRWF:SetFont()
	RaidWarningFrame:SetFont(fonts[self.db.profile.font], fontsizes[self.db.profile.fontsize], fontoutlines[self.db.profile.fontoutline])
end


function ImprovedRWF:GenerateAddMessage()
	local info = self.db.profile.usecustomcolour and self.db.profile.customcolour or ChatTypeInfo["RAID_WARNING"]
	if self.db.profile.addname then
		if Teknicolor and self.db.profile.colourname then
			return function(message, sender) 
				if Teknicolor.nametable[arg2] then
					RaidWarningFrame:AddMessage(Teknicolor.nametable[sender] .. ": " .. message, info.r, info.g, info.b, 1.0)
				else
					RaidWarningFrame:AddMessage(sender .. ": " .. message, info.r, info.g, info.b, 1.0)
				end
			end
		else
			return function(message, sender) RaidWarningFrame:AddMessage(sender .. ": " .. message, info.r, info.g, info.b, 1.0) end
		end
	end	

	return function(message) RaidWarningFrame:AddMessage(message, info.r, info.g, info.b, 1.0) end
end

function ImprovedRWF:GenerateIsSpam()
	if self.db.profile.supressown then
		if BigWigsBossBlock and BigWigsBossBlock:IsHooked("RaidWarningFrame_OnEvent") then
			local bbdb = BigWigsBossBlock.db
			return function(message, sender) return (PLAYER == sender) or (bbdb.profile.hideraidwarn and BigWigsBossBlock:IsSpam(message)) end
		else
			return function(message, sender) return PLAYER == sender end
		end
	else
		if BigWigsBossBlock and BigWigsBossBlock:IsHooked("RaidWarningFrame_OnEvent") then
			local bbdb = BigWigsBossBlock.db
			return function(message) return bbdb.profile.hideraidwarn and BigWigsBossBlock:IsSpam(message) end
		end
	end

	return __DoNothing
end

function ImprovedRWF:GeneratePlaySound()
	if self.db.profile.sound then
		return PlaySound
	end

	return __DoNothing
end


function ImprovedRWF:Test(message)
	-- back up arg2
	local a2 = arg2

	-- add the test message
	arg2 = PLAYER
	self:RaidWarningFrame_OnEvent("CHAT_MSG_RAID_WARNING", message)

	-- restore arg2
	arg2 = a2
end


--------------------------------
--      Hooked Functions      --
--------------------------------

function ImprovedRWF:_RaidWarningFrame_OnEvent_SupressAll(event, message)
	if event ~= "CHAT_MSG_RAID_WARNING" then
		self.hooks.RaidWarningFrame_OnEvent.orig(event, message)
	end
end

function ImprovedRWF:_RaidWarningFrame_OnEvent(event, message)
	if event == "CHAT_MSG_RAID_WARNING" then
		if not message or _IsSpam(message, arg2) then return end

		_AddMessage(message, arg2)
		_PlaySound("RaidWarning")

	else
		self.hooks.RaidWarningFrame_OnEvent.orig(event, message)
	end
end
